@@ -19,6 +19,7 @@
  */
 
 #include "application.h"
+#include "dialog_new_database.h"
 #include <libgnome/gnome-help.h> //For gnome_help_display
 #include "config.h" //For VERSION.
 #include <cstdio>
@@ -36,9 +37,12 @@ App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml
   refGlade->get_widget("bakery_vbox", m_pBoxTop);
   refGlade->get_widget_derived("vbox_frame", m_pFrame); //This one is derived. There's a lot happening here.
   refGlade->get_widget("label_hint", m_pStatus);
-  
+ 
   add_mime_type("application/x-glom"); //TODO: make this actually work - we need to register it properly.
   
+  //Hide the toolbar because it doesn't contain anything useful for this app.
+  m_HandleBox_Toolbar.hide();
+  
   show_all_children();
 }
 
@@ -47,7 +51,7 @@ App_Glom::~App_Glom()
 
 }
 
-void App_Glom::init()
+bool App_Glom::init(const Glib::ustring& document_uri)
 {
   type_vecStrings vecAuthors;
   vecAuthors.push_back("Murray Cumming <murrayc@murrayc.com>");
@@ -57,12 +61,25 @@ void App_Glom::init()
 
   //m_pFrame->set_shadow_type(Gtk::SHADOW_IN);
 
-  Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
-  if(pDocument->get_connection_database().empty()) //If it is a new (default) document.
+  //Hide the toolbar because it doesn't contain anything useful for this app.
+  //m_HandleBox_Toolbar.hide();
+  
+  if(document_uri.empty())
   {
-    offer_new_or_existing();
+    Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
+    if(pDocument->get_connection_database().empty()) //If it is a new (default) document.
+    {
+      return offer_new_or_existing();
+    }
+  }
+  else
+  {
+    bool test = open_document(document_uri);
+    if(!test)
+      return offer_new_or_existing();
   }
   
+  return true;
   //show_all();
 }
 
@@ -75,19 +92,94 @@ void App_Glom::init_layout()
   Gtk::MenuBar* pMenuBar = static_cast<Gtk::MenuBar*>(m_refUIManager->get_widget("/Bakery_MainMenu"));
   m_pBoxTop->pack_start(*pMenuBar, Gtk::PACK_SHRINK);
 
-  Gtk::Toolbar* pToolBar = static_cast<Gtk::Toolbar*>(m_refUIManager->get_widget("/Bakery_ToolBar"));
-  m_HandleBox_Toolbar.add(*pToolBar);
-  m_HandleBox_Toolbar.show();
+  //Do not create the toolbar because it doesn't contain anything useful for this app.
+  //Gtk::Toolbar* pToolBar = static_cast<Gtk::Toolbar*>(m_refUIManager->get_widget("/Bakery_ToolBar"));
+  //m_HandleBox_Toolbar.add(*pToolBar);
+  //m_HandleBox_Toolbar.show();
 
   add_accel_group(m_refUIManager->get_accel_group());
 
-  m_pBoxTop->pack_start(m_HandleBox_Toolbar, Gtk::PACK_SHRINK);
+  //m_pBoxTop->pack_start(m_HandleBox_Toolbar, Gtk::PACK_SHRINK);
 
   //Add placeholder, to be used by add():
   //m_pBoxTop->pack_start(m_VBox_PlaceHolder);
   //m_VBox_PlaceHolder.show();
 }
 
+void App_Glom::init_toolbars()
+{
+  //We override this because
+  //a) We don't want a toolbar, and
+  //b) The default toolbar layout has actions that we don't have.
+  
+/*
+  //Build part of the menu structure, to be merged in by using the "PH" placeholders:
+  static const Glib::ustring ui_description =
+    "<ui>"
+    "  <toolbar name='Bakery_ToolBar'>"
+    "    <placeholder name='Bakery_ToolBarItemsPH'>"
+    "      <toolitem action='BakeryAction_File_New' />"
+    "      <toolitem action='BakeryAction_File_Open' />"
+    "      <toolitem action='BakeryAction_File_Save' />"
+    "    </placeholder>"
+    "  </toolbar>"
+    "</ui>";
+
+  add_ui_from_string(ui_description);
+*/
+}
+
+void App_Glom::init_menus_file()
+{
+  //Overridden to remove the Save and Save-As menu items,
+  //because all changes are saved immediately and automatically.
+  
+  // File menu
+
+  //Build actions:
+  m_refFileActionGroup = Gtk::ActionGroup::create("BakeryFileActions");
+
+  m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_Menu_File", gettext("_File")));
+  m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_Menu_File_RecentFiles", gettext("_Recent Files")));
+
+  //File actions
+  m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_File_New", Gtk::Stock::NEW),
+                        sigc::mem_fun((App&)*this, &App::on_menu_file_new));
+  m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_File_Open", Gtk::Stock::OPEN),
+                        sigc::mem_fun((App_WithDoc&)*this, &App_WithDoc::on_menu_file_open));
+
+  m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_File_Close", Gtk::Stock::CLOSE),
+                        sigc::mem_fun((App_WithDoc&)*this, &App_WithDoc::on_menu_file_close));
+  m_refFileActionGroup->add(Gtk::Action::create("BakeryAction_File_Exit", Gtk::Stock::QUIT),
+                        sigc::mem_fun((App&)*this, &App::on_menu_file_exit));
+
+  m_refUIManager->insert_action_group(m_refFileActionGroup);
+
+  //Build part of the menu structure, to be merged in by using the "PH" placeholders:
+  static const Glib::ustring ui_description =
+    "<ui>"
+    "  <menubar name='Bakery_MainMenu'>"
+    "    <placeholder name='Bakery_MenuPH_File'>"
+    "      <menu action='BakeryAction_Menu_File'>"
+    "        <menuitem action='BakeryAction_File_New' />"
+    "        <menuitem action='BakeryAction_File_Open' />"
+    "        <menu action='BakeryAction_Menu_File_RecentFiles'>"
+    "        </menu>"
+    "        <separator/>"
+    "        <menuitem action='BakeryAction_File_Close' />"
+    "        <menuitem action='BakeryAction_File_Exit' />"
+    "      </menu>"
+    "    </placeholder>"
+    "  </menubar>"
+    "</ui>";
+
+  //Add menu:
+  add_ui_from_string(ui_description);
+
+  //Add recent-files submenu:
+  init_menus_file_recentfiles("/Bakery_MainMenu/Bakery_MenuPH_File/BakeryAction_Menu_File/BakeryAction_Menu_File_RecentFiles");
+}
+
 void App_Glom::init_menus()
 {
   init_menus_file();
@@ -95,40 +187,34 @@ void App_Glom::init_menus()
 
   //Build actions:
   m_refActionGroup_Others = Gtk::ActionGroup::create("GlomOthersActions");
-
-  //We save some action in m_listDeveloperActions - these will all be enabled/disabled when the userlevel changes.
-  m_listDeveloperActions.push_back(m_action_save);
-  m_listDeveloperActions.push_back(m_action_saveas);
   
   //"Navigate" menu:
   m_refActionGroup_Others->add( Gtk::Action::create("Glom_Menu_Navigate", gettext("_Navigate")) );
 
-  Glib::RefPtr<Gtk::Action> action = Gtk::Action::create("GlomAction_Menu_Navigate_Database", gettext("_Database"));
-  m_listDeveloperActions.push_back(action);
-  m_refActionGroup_Others->add(action,
-                        sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Navigate_Database) );
+//  Glib::RefPtr<Gtk::Action> action = Gtk::Action::create("GlomAction_Menu_Navigate_Database", gettext("_Database"));
+//  m_listDeveloperActions.push_back(action);
+//  m_refActionGroup_Others->add(action,
+//                        sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Navigate_Database) );
 
 
-  action = Gtk::Action::create("GlomAction_Menu_Navigate_Table", gettext("_Table"));
-  m_listWithDatabaseActions.push_back(action); //Remember it so we can disable/enable it when a database has been selected.
+  Glib::RefPtr<Gtk::Action> action = Gtk::Action::create("GlomAction_Menu_Navigate_Table", gettext("_Table"));
   m_refActionGroup_Others->add(action,
                         sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_Navigate_Table) );
 
   //"UserLevel" menu:
-  m_refActionGroup_Others->add(Gtk::Action::create("Glom_Menu_UserLevel", gettext("_User Level")));
+  m_refActionGroup_Others->add(Gtk::Action::create("Glom_Menu_userlevel", gettext("_User Level")));
   Gtk::RadioAction::Group group_userlevel;
 
-  m_action_menu_userlevel_developer = Gtk::RadioAction::create(group_userlevel, "GlomAction_Menu_UserLevel_Developer", gettext("_Developer"));
+  m_action_menu_userlevel_developer = Gtk::RadioAction::create(group_userlevel, "GlomAction_Menu_userlevel_Developer", gettext("_Developer"));
   m_refActionGroup_Others->add(m_action_menu_userlevel_developer,
-                        sigc::bind( sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_UserLevel_Developer), m_action_menu_userlevel_developer) );
+                        sigc::mem_fun(*this, &App_Glom::on_menu_userlevel_developer) );
 
-  m_action_menu_userlevel_operator =  Gtk::RadioAction::create(group_userlevel, "GlomAction_Menu_UserLevel_Operator", gettext("_Operator"));
+  m_action_menu_userlevel_operator =  Gtk::RadioAction::create(group_userlevel, "GlomAction_Menu_userlevel_Operator", gettext("_Operator"));
   m_refActionGroup_Others->add(m_action_menu_userlevel_operator,
-                        sigc::bind( sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_UserLevel_Operator), m_action_menu_userlevel_operator) );
+                          sigc::mem_fun(*this, &App_Glom::on_menu_userlevel_operator) );
 
   //"Mode" menu:
   action =  Gtk::Action::create("Glom_Menu_Mode", gettext("_Mode"));
-  m_listWithDatabaseActions.push_back(action);
   m_refActionGroup_Others->add(action);
   Gtk::RadioAction::Group group_mode;
 
@@ -159,6 +245,10 @@ void App_Glom::init_menus()
   m_listDeveloperActions.push_back(action);
   m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_layout));
 
+  action = Gtk::Action::create("GlomAction_Menu_Developer_RecreateStructure", gettext("_Recreate Database Structure"));
+  m_listDeveloperActions.push_back(action);
+  m_refActionGroup_Others->add(action, sigc::mem_fun(*m_pFrame, &Frame_Glom::on_menu_developer_recreate_structure));
+
   
   m_refUIManager->insert_action_group(m_refActionGroup_Others);
 
@@ -168,22 +258,23 @@ void App_Glom::init_menus()
     "  <menubar name='Bakery_MainMenu'>"
     "    <placeholder name='Bakery_MenuPH_Others'>"
     "      <menu action='Glom_Menu_Navigate'>"
-    "        <menuitem action='GlomAction_Menu_Navigate_Database' />"
     "        <menuitem action='GlomAction_Menu_Navigate_Table' />"
     "     </menu>"
     "      <menu action='Glom_Menu_Mode'>"
     "        <menuitem action='GlomAction_Menu_Mode_Data' />"
     "        <menuitem action='GlomAction_Menu_Mode_Find' />"
     "      </menu>"
-    "      <menu action='Glom_Menu_UserLevel'>"
-    "        <menuitem action='GlomAction_Menu_UserLevel_Developer' />"
-    "        <menuitem action='GlomAction_Menu_UserLevel_Operator' />"
+    "      <menu action='Glom_Menu_userlevel'>"
+    "        <menuitem action='GlomAction_Menu_userlevel_Developer' />"
+    "        <menuitem action='GlomAction_Menu_userlevel_Operator' />"
     "      </menu>"
     "      <menu action='Glom_Menu_Developer'>"
     "        <menuitem action='GlomAction_Menu_Developer_Fields' />"
     "        <menuitem action='GlomAction_Menu_Developer_Relationships' />"    
     "        <menuitem action='GlomAction_Menu_Developer_Layout' />"
     "        <menuitem action='GlomAction_Menu_Developer_Users' />"
+    "        <separator /> "
+    "        <menuitem action='GlomAction_Menu_Developer_RecreateStructure' />"
     "      </menu>"
     "    </placeholder>"
     "  </menubar>"
@@ -193,10 +284,19 @@ void App_Glom::init_menus()
   add_ui_from_string(ui_description);
 
   init_menus_help();
-
-  on_database_selected(false);
 }
 
+void App_Glom::on_menu_userlevel_developer()
+{
+  if(m_pFrame)
+    m_pFrame->on_menu_userlevel_Developer(m_action_menu_userlevel_developer);
+}
+
+void App_Glom::on_menu_userlevel_operator()
+{
+  if(m_pFrame)
+    m_pFrame->on_menu_userlevel_Operator(m_action_menu_userlevel_operator);
+}
 
 Bakery::App* App_Glom::new_instance() //Override
 {
@@ -252,14 +352,21 @@ void App_Glom::on_document_load()
         //The ConnectionPool will now use these every time it tries to connect.
         connection_pool->set_host(pDocument->get_connection_server());
         connection_pool->set_user(pDocument->get_connection_user());
-
+        connection_pool->set_database(pDocument->get_connection_database());
+        
         connection_pool->set_ready_to_connect(); //Box_DB::connect_to_server() will now attempt the connection-> Shared instances of m_Connection will also be usable.
+
+        //Attempt to connect to the specified database:
+        bool test = m_pFrame->connection_request_password_and_attempt();
+        if(!test)
+          return; //Failed. TODO: Close the document?
       }
 
       //Switch to operator mode when opening new documents:
       pDocument->set_userlevel(AppState::USERLEVEL_OPERATOR);
 
-      m_pFrame->do_menu_Navigate_Database(false); //false = don't show list, just open the current database after connecting.
+      //Open default table, or show list of tables instead:
+      m_pFrame->do_menu_Navigate_Table(true /* open the default if there is one */);    
     }
   }
 }
@@ -297,22 +404,12 @@ void App_Glom::on_userlevel_changed(AppState::userlevels userlevel)
   }
 }
 
-void App_Glom::on_database_selected(bool database_selected)
-{
-  //Disable/Enable developer actions:
-  for(type_listActions::iterator iter = m_listWithDatabaseActions.begin(); iter != m_listWithDatabaseActions.end(); ++iter)
-  {
-    Glib::RefPtr<Gtk::Action> action = *iter;
-     action->set_sensitive ( database_selected );
-  }
-}
-
 Glib::RefPtr<Gtk::UIManager> App_Glom::get_ui_manager()
 {
   return m_refUIManager;
 }
 
-void App_Glom::offer_new_or_existing()
+bool App_Glom::offer_new_or_existing()
 {
   //Offer to load an existing document, or start a new one.
   Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_existing_or_new");
@@ -320,23 +417,120 @@ void App_Glom::offer_new_or_existing()
   refXml->get_widget("dialog_existing_or_new", dialog);
   dialog->set_transient_for(*this);
   
-  guint response_id = dialog->run();
+  int response_id = dialog->run();
   delete dialog;
   dialog = 0;
   
   if(response_id ==1) //Open
   {
     on_menu_file_open();
+
+    //Check that a document was opened:
+    Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+    if(document->get_file_uri().empty())
+    {
+      //Ask again:
+      return offer_new_or_existing();
+    }
   }
   else if(response_id == 2) //New
   {
-    //There is already an empty document, but this allows the user to fill it by connecting and creating tables:
-    m_pFrame->do_menu_Navigate_Database();
+    //Each document must have a location, so ask the user for one.
+    offer_saveas();
+
+    //Check that the document was given a location:
+    Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
+    if(!document->get_file_uri().empty())
+    {
+      //Make sure that the user can do something with his new document:
+      document->set_userlevel(AppState::USERLEVEL_DEVELOPER);
+
+      //Each new document must have an associated new database,
+      //so ask the user for the name of one to create:
+      Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_new_database");
+      if(refXml)
+      {
+        Dialog_NewDatabase* dialog = 0;
+        refXml->get_widget_derived("dialog_new_database", dialog);
+        if(dialog)
+        {
+          //Set suitable defaults:
+          const Glib::ustring filename = document->get_name(); //Get the filename without the path and extension.
+          dialog->set_input(filename, Base_DB::util_title_from_string( filename ) );
+
+          bool keep_asking = true;
+          while(keep_asking)
+          {
+            int response = dialog->run(); //TODO: Allow the user to cancel.
+            dialog->hide();
+            if(response == Gtk::RESPONSE_OK)
+            {
+              Glib::ustring db_name;
+              Glib::ustring db_title;
+              dialog->get_input(db_name, db_title);
+
+              //Prefix glom_ to the database name, so it's more obvious
+              //for the system administrator.
+              //This database name should never be user-visible again, either prefixed or not prefixed.
+              db_name = "glom_" + db_name;
+
+              if(!db_name.empty()) //The dialog prevents this anyway.
+              {
+                bool db_created = m_pFrame->create_database(db_name);
+                if(db_created)
+                {
+                  keep_asking = false;
+                
+                  document->set_connection_database(db_name); //Select the database that was just created.
+
+                  ConnectionPool* connection_pool = ConnectionPool::get_instance();
+                  if(connection_pool)
+                  {
+                    connection_pool->set_database(db_name); //The rest has been set while creating the database.
+                  }
+
+                  document->set_database_title(db_title);
+                  m_pFrame->set_databases_selected(db_name);
+                }
+                else
+                {
+                  return false;
+                }
+              }
+              else
+              {
+                g_warning(" App_Glom::offer_new_or_existing(): db_name is empty.");
+                //And ask again, by going back to the start of the while() loop.
+              }
+            } /* if(response) */
+            else
+            {
+              return false; //The user cancelled.
+            }            
+          } /* while() */
+          
+          return true; //File successfully created.
+        }
+       }
+  
+      return false; //Creation of new document failed.
+    }
+    else
+    {
+      //Ask again:
+      return offer_new_or_existing();
+    }
+  }
+  else if(response_id == Gtk::RESPONSE_CANCEL)
+  {
+    return false; //close the window to close the application, because they need to choose a new or existing document.
   }
   else
   {
     //Do nothing. TODO: Do something?
   }
+
+  return true;
 }
 
 void App_Glom::set_mode_data()
@@ -384,10 +578,3 @@ AppState::userlevels App_Glom::get_userlevel() const
   else
     return AppState::USERLEVEL_DEVELOPER; //This should never happen.
 }
-
-
-
-
-
-
-
