@@ -18,9 +18,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
+//We use Python for calculated fields.
+#include <Python.h> //Include it before anything else to avoid "_POSIX_C_SOURCE redefined".
+
 //#include <gnome.h>
 #include <gtkmm/main.h>
 #include <libgnome/gnome-init.h> // For gnome_program_init().
+#include <libintl.h>
+
+
+
 #include "config.h" //For VERSION.
 
 #include "application.h"
@@ -36,6 +43,20 @@ main(int argc, char* argv[])
 
   Gnome::Gda::init("glom", VERSION, argc, argv);
 
+  //Get command-line parameters, if any:
+  Glib::ustring input_uri;
+  if(argc > 1 )
+  {
+    input_uri = argv[1];
+
+    //Ignore arguements starting with "--", or "-".
+    if(input_uri.size() && input_uri[0] == '-')
+      input_uri = Glib::ustring();
+  }
+
+  //We use python for calculated-fields:
+  Py_Initialize();
+  
   try
   {
     //Initialize gnome_program, so that we can use gnome_help_display().    
@@ -51,15 +72,23 @@ main(int argc, char* argv[])
     refXml->get_widget_derived("window_main", pApp_Glom);
 
     pApp_Glom->set_command_line_args(argc, argv);
-    pApp_Glom->init(); //Sets it up and shows it.
 
-    Gtk::Main::run();
+    bool test = pApp_Glom->init(input_uri); //Sets it up and shows it.
+    if(test) //The user could cancel the offer of a new or existing database.
+    {
+      Gtk::Main::run();
+    }
+    else
+      delete pApp_Glom;
   }
   catch(const std::exception& ex)
   {
     std::cerr << "Glom: exception: \n" << ex.what() << std::endl;
   }
 
+  //We use python for calculated-fields:
+  Py_Finalize();
+  
   return 0;
 }
 
