@@ -47,7 +47,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
     if(dialog)
     {
       m_pDialogLayout = dialog;
-      m_pDialogLayout->signal_hide().connect( sigc::mem_fun(*this, &Box_Data::on_dialog_layout_hide) );
+      m_pDialogLayout->signal_hide().connect( sigc::mem_fun(static_cast<Box_Data&>(*this), &Box_Data::on_dialog_layout_hide) );
     }
   }
   
@@ -204,9 +204,7 @@ void Box_Data_Details::fill_from_database()
 
               //Field value:
               Gnome::Gda::Value value = result->get_value_at(i, row_number);
-              g_warning("debug 1");
               m_FlowTable.set_field_value(field, value);
-                g_warning("debug 2");
             }
           }
         }
@@ -617,6 +615,11 @@ void Box_Data_Details::do_lookups(const Field& field_changed, const Gnome::Gda::
 
 }
 
+void Box_Data_Details::on_userlevel_changed(AppState::userlevels user_level)
+{
+  m_FlowTable.set_design_mode( user_level == AppState::USERLEVEL_DEVELOPER );
+}
+
 
 
 
