@@ -349,6 +349,11 @@ Glib::ustring AddDel::get_value_selected(guint col)
   return strValue;
 }
 
+Gnome::Gda::Value AddDel::get_value_selected_as_value(guint col)
+{
+  return get_value_as_value(get_item_selected(), col);
+}
+
 Gtk::TreeModel::iterator AddDel::get_item_selected()
 {
   Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_TreeView.get_selection();
@@ -374,6 +379,13 @@ Gtk::TreeModel::iterator AddDel::get_row(const Glib::ustring& key)
   return  m_refListStore->children().end();
 }
 
+bool AddDel::select_item(const Gtk::TreeModel::iterator& iter)
+{
+  guint col_first = 0;
+  get_model_column_index(0, col_first);
+  return select_item(iter, col_first);
+}
+
 bool AddDel::select_item(const Gtk::TreeModel::iterator& iter, guint column, bool start_editing)
 {
   if(!m_refListStore)
@@ -403,7 +415,7 @@ bool AddDel::select_item(const Gtk::TreeModel::iterator& iter, guint column, boo
            g_warning("AddDel::select_item:TreeViewColumn not found.");
       }
       else
-           g_warning("AddDel::select_item:TreeViewColumn index not found.");
+           g_warning("AddDel::select_item:TreeViewColumn index not found. column=%d", column);
     }
 
     bResult = true;
@@ -780,6 +792,11 @@ void AddDel::set_value(const Gtk::TreeModel::iterator& iter, guint col, bool bVa
   }
 }
 
+void AddDel::set_value_selected(guint col, const Gnome::Gda::Value& value)
+{
+  set_value(get_item_selected(), col, value);
+}
+
 void AddDel::remove_all_columns()
 {
   m_ColumnTypes.clear();
@@ -842,6 +859,15 @@ guint AddDel::add_column(const Glib::ustring& strTitle, const Glib::ustring& col
   return add_column(column_info);
 }
 
+Glib::ustring AddDel::get_column_field(guint column_index) const
+{
+  Glib::ustring result;
+  if(column_index < m_ColumnTypes.size())
+    result = m_ColumnTypes[column_index].m_name;
+
+  return result;
+}
+
 bool AddDel::get_prevent_user_signals() const
 {
   return m_bPreventUserSignals;
@@ -1016,6 +1042,9 @@ Glib::ustring AddDel::treeview_get_key(const Gtk::TreeModel::iterator& row)
 
 void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int model_column_index)
 {
+  if(path_string.empty())
+    return;
+    
   Gtk::TreePath path(path_string);
 
   //Get the row from the path:
@@ -1088,6 +1117,9 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
 
 void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index)
 {
+  if(path_string.empty())
+    return;
+  
   Gtk::TreePath path(path_string);
 
   //Get the row from the path:
